import { NormalizeLocale } from './resolve-accept-language';
/** Lookup list used to match the preferred locale based on the value of an `Accept-Language` HTTP header. */
export default class LookupList<TLocales extends readonly string[]> {
    /** The list of locales used to get the match during the lookup. */
    private localeList;
    /**
     * Data object where the properties are quality (in string format) and their values a set containing locale
     * identifiers using the `language`-`country` format and ISO 639-1 alpha-2 language code.
     */
    private localesAndLanguagesByQuality;
    /**
     * Data object where the properties are quality (in string format) and their value a set of ISO 639-1 alpha-2
     * language code.
     */
    private relatedLocaleLanguagesByQuality;
    /**
     * Create a new `LookupList` object.
     *
     * @param acceptLanguageHeader - The value of an HTTP request `Accept-Language` header (also known as a "language priority list").
     * @param locales - An array of locale identifiers. The order will be used for matching where the first identifier will be more
     * likely to be matched than the last identifier.
     */
    constructor(acceptLanguageHeader: string, locales: TLocales extends string[] ? TLocales[number][] : TLocales, defaultLocale: TLocales[number]);
    /**
     * Get the top locale-based match if available.
     *
     * @returns The top locale-based match or undefined when there is no match.
     */
    getLocaleBasedMatch(): NormalizeLocale<TLocales[number]> | undefined;
    /**
     * Get the language-based match if available.
     *
     * @returns The language-based match or undefined when there is no match.
     */
    getLanguageBasedMatch(): NormalizeLocale<TLocales[number]> | undefined;
    /**
     * Get the related-locale-based match if available.
     *
     * @returns The related-locale-based match or undefined when there is no match.
     */
    getRelatedLocaleBasedMatch(): NormalizeLocale<TLocales[number]> | undefined;
    /**
     * Add a language in the data object matching its quality.
     *
     * @param quality - The HTTP header's quality factor associated with a language.
     * @param languageCode - An ISO 639-1 alpha-2 language code.
     */
    private addLanguage;
    /**
     * Add a locale in the data object matching its quality.
     *
     * @param quality - The HTTP header's quality factor associated with a locale.
     * @param identifier - A locale identifier using the BCP 47 `language`-`country` case-normalized format.
     */
    private addLocale;
    /**
     * Add a related locale's language in the data object matching its quality.
     *
     * @param quality - The HTTP header's quality factor associated with a related locale's language.
     * @param languageCode - An ISO 639-1 alpha-2 language code.
     */
    private addRelatedLocaleLanguage;
    /**
     * Get a directive object from a directive string.
     *
     * @param directiveString - The string representing a directive, extracted from the HTTP header.
     *
     * @returns A `Directive` object or `undefined` if the string's format is invalid.
     */
    private getDirective;
    /**
     * Get a match from a data object.
     *
     * @param dataObject - A data object.
     *
     * @returns A match or undefined when there is no match.
     */
    private getMatch;
}
