/** The type matches. */
export type MatchType = 'localeBased' | 'languageBased' | 'relatedLocaleBased' | 'defaultLocale';
/** Match type enumeration. */
export declare const MATCH_TYPES: {
    readonly [K in MatchType]: K;
};
/** Type to normalize the locale format. */
export type NormalizeLocale<Remainder extends string> = Remainder extends `${infer LanguageCode}-${infer CountryCode}` ? `${Lowercase<LanguageCode>}-${Uppercase<CountryCode>}` : Remainder;
/** Resolve the preferred locale from an HTTP `Accept-Language` header. */
export declare class ResolveAcceptLanguage<TLocales extends readonly string[] = string[]> {
    /** The default locale. */
    private defaultLocale;
    /** The locale-based match, if applicable. */
    private localeBasedMatch;
    /** The language-based match, if applicable. */
    private languageBasedMatch;
    /** The related-locale-based match, if applicable. */
    private relatedLocaleBasedMatch;
    /**
     * Create a new `ResolveAcceptLanguage` object.
     *
     * All locale identifiers provided as parameters must following the BCP 47 `language`-`country` (case insensitive).
     *
     * @param acceptLanguageHeader - The value of an HTTP request `Accept-Language` header (also known as a "language priority list").
     * @param locales - An array of locale identifiers. The order will be used for matching where the first identifier will be more
     * likely to be matched than the last identifier.
     */
    constructor(acceptLanguageHeader: string, locales: TLocales extends string[] ? TLocales[number][] : TLocales, defaultLocale: TLocales[number]);
    /**
     * Get the type of match.
     *
     * @returns The type of match.
     */
    getMatchType(): MatchType;
    /**
     * Get the matching locale.
     *
     * @returns The matching locale.
     */
    getMatch(): NormalizeLocale<TLocales[number]>;
}
/**
 * Resolve the preferred locale from an HTTP `Accept-Language` header.
 *
 * All locale identifiers provided as parameters must following the BCP 47 `language`-`country` (case insensitive).
 *
 * @param acceptLanguageHeader - The value of an HTTP request `Accept-Language` header (also known as a "language priority list").
 * @param locales - An array of locale identifiers that must include the default locale. The order will be used for matching where
 * the first identifier will be more likely to be matched than the last identifier.
 * @param defaultLocale - The default locale identifier when no match is found.
 *
 * @returns The locale identifier which was the best match, in case-normalized format.
 *
 * @example
 * // returns 'fr-CA'
 * resolveAcceptLanguage(
 *   'fr-CA;q=0.01,en-CA;q=0.1,en-US;q=0.001',
 *   ['en-US', 'fr-CA'],
 *   'en-US'
 * )
 */
declare const resolveAcceptLanguage: <TLocales extends readonly string[]>(acceptLanguageHeader: string, locales: TLocales extends string[] ? TLocales[number][] : TLocales, defaultLocale: TLocales[number]) => NormalizeLocale<TLocales[number]>;
export default resolveAcceptLanguage;
