var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
import Locale from './locale';
import LocaleList from './locale-list';
/** Lookup list used to match the preferred locale based on the value of an `Accept-Language` HTTP header. */
var LookupList = /** @class */ (function () {
    /**
     * Create a new `LookupList` object.
     *
     * @param acceptLanguageHeader - The value of an HTTP request `Accept-Language` header (also known as a "language priority list").
     * @param locales - An array of locale identifiers. The order will be used for matching where the first identifier will be more
     * likely to be matched than the last identifier.
     */
    function LookupList(acceptLanguageHeader, locales, defaultLocale) {
        var _this = this;
        /**
         * Data object where the properties are quality (in string format) and their values a set containing locale
         * identifiers using the `language`-`country` format and ISO 639-1 alpha-2 language code.
         */
        this.localesAndLanguagesByQuality = {};
        /**
         * Data object where the properties are quality (in string format) and their value a set of ISO 639-1 alpha-2
         * language code.
         */
        this.relatedLocaleLanguagesByQuality = {};
        // Put the default locale first so that it will be more likely to be matched.
        this.localeList = new LocaleList(__spreadArray([
            defaultLocale
        ], locales.filter(function (locale) { return locale !== defaultLocale; }), true));
        var directives = acceptLanguageHeader
            .split(',')
            .map(function (directiveString) { return _this.getDirective(directiveString.trim()); })
            .filter(function (directive) { return directive !== undefined; });
        // Check if 'es-419' exists in the directives
        var es419DirectiveIndex = directives.findIndex(function (directive) { return directive.locale === 'es-419'; });
        if (es419DirectiveIndex >= 0) {
            // Remove 'es-419' from the directive.
            var es419Directive_1 = directives[es419DirectiveIndex];
            directives.splice(es419DirectiveIndex, 1);
            // Replace `es-419` by the common Latin American spanish variants supported by browsers.
            var latinAmericanLocales = [
                'es-AR', // Spanish Argentina
                'es-CL', // Spanish Chile
                'es-CO', // Spanish Colombia
                'es-CR', // Spanish Costa Rica
                'es-HN', // Spanish Honduras
                'es-MX', // Spanish Mexico
                'es-PE', // Spanish Peru
                'es-US', // Spanish United States
                'es-UY', // Spanish Uruguay
                'es-VE', // Spanish Venezuela
            ];
            latinAmericanLocales.forEach(function (locale) {
                directives.push({
                    languageCode: 'es',
                    locale: locale,
                    quality: es419Directive_1.quality,
                });
            });
        }
        for (var _i = 0, directives_1 = directives; _i < directives_1.length; _i++) {
            var directive = directives_1[_i];
            var locale = directive.locale, languageCode = directive.languageCode, quality = directive.quality;
            // If the language is not supported, skip to the next match.
            if (!this.localeList.languages.has(languageCode)) {
                continue;
            }
            // If there is no country code (while the language is supported), add the language preference.
            if (!locale) {
                this.addLanguage(quality, languageCode);
                continue;
            }
            // If the locale is not supported, but the locale's language is, add to locale language preference.
            if (!this.localeList.locales.has(locale) && this.localeList.languages.has(languageCode)) {
                this.addRelatedLocaleLanguage(quality, languageCode);
                continue;
            }
            // If the locale is supported, add the locale preference.
            this.addLocale(quality, locale);
        }
    }
    /**
     * Get the top locale-based match if available.
     *
     * @returns The top locale-based match or undefined when there is no match.
     */
    LookupList.prototype.getLocaleBasedMatch = function () {
        var match = this.getMatch(this.localesAndLanguagesByQuality);
        return match && Locale.isLocale(match)
            ? match
            : undefined;
    };
    /**
     * Get the language-based match if available.
     *
     * @returns The language-based match or undefined when there is no match.
     */
    LookupList.prototype.getLanguageBasedMatch = function () {
        var match = this.getMatch(this.localesAndLanguagesByQuality);
        return match && !Locale.isLocale(match)
            ? this.localeList.objects.find(function (locale) { return locale.languageCode === match; })
                .identifier
            : undefined;
    };
    /**
     * Get the related-locale-based match if available.
     *
     * @returns The related-locale-based match or undefined when there is no match.
     */
    LookupList.prototype.getRelatedLocaleBasedMatch = function () {
        var match = this.getMatch(this.relatedLocaleLanguagesByQuality);
        return match
            ? this.localeList.objects.find(function (locale) { return locale.languageCode === match; })
                .identifier
            : undefined;
    };
    /**
     * Add a language in the data object matching its quality.
     *
     * @param quality - The HTTP header's quality factor associated with a language.
     * @param languageCode - An ISO 639-1 alpha-2 language code.
     */
    LookupList.prototype.addLanguage = function (quality, languageCode) {
        if (!this.localesAndLanguagesByQuality[quality]) {
            this.localesAndLanguagesByQuality[quality] = new Set();
        }
        this.localesAndLanguagesByQuality[quality].add(languageCode);
    };
    /**
     * Add a locale in the data object matching its quality.
     *
     * @param quality - The HTTP header's quality factor associated with a locale.
     * @param identifier - A locale identifier using the BCP 47 `language`-`country` case-normalized format.
     */
    LookupList.prototype.addLocale = function (quality, identifier) {
        if (!this.localesAndLanguagesByQuality[quality]) {
            this.localesAndLanguagesByQuality[quality] = new Set();
        }
        this.localesAndLanguagesByQuality[quality].add(identifier);
    };
    /**
     * Add a related locale's language in the data object matching its quality.
     *
     * @param quality - The HTTP header's quality factor associated with a related locale's language.
     * @param languageCode - An ISO 639-1 alpha-2 language code.
     */
    LookupList.prototype.addRelatedLocaleLanguage = function (quality, languageCode) {
        if (!this.relatedLocaleLanguagesByQuality[quality]) {
            this.relatedLocaleLanguagesByQuality[quality] = new Set();
        }
        this.relatedLocaleLanguagesByQuality[quality].add(languageCode);
    };
    /**
     * Get a directive object from a directive string.
     *
     * @param directiveString - The string representing a directive, extracted from the HTTP header.
     *
     * @returns A `Directive` object or `undefined` if the string's format is invalid.
     */
    LookupList.prototype.getDirective = function (directiveString) {
        /**
         * The regular expression is excluding certain directives due to the inability to configure those options in modern
         * browsers today (also those options seem unpractical):
         *
         * - The wildcard character "*", as per RFC 2616 (section 14.4), should match any unmatched language tag.
         * - Language tags that starts with a wildcard (e.g., "*-CA") should match the first supported locale of a country.
         * - A quality value equivalent to "0", as per RFC 2616 (section 3.9), should be considered as "not acceptable".
         * - We hardcode the support for the `419` UN M49 code (as country code) representing Latin America to support `es-419`.
         */
        var directiveMatch = directiveString.match(/^((?<matchedLanguageCode>([a-z]{2}))(-(?<matchedCountryCode>[a-z]{2}|419))?)(;q=(?<matchedQuality>(1(\.0{0,3})?)|(0(\.\d{0,3})?)))?$/i);
        if (!(directiveMatch === null || directiveMatch === void 0 ? void 0 : directiveMatch.groups)) {
            return undefined; // No regular expression match.
        }
        var _a = directiveMatch.groups, matchedLanguageCode = _a.matchedLanguageCode, matchedCountryCode = _a.matchedCountryCode, matchedQuality = _a.matchedQuality;
        var languageCode = matchedLanguageCode.toLowerCase();
        var countryCode = matchedCountryCode ? matchedCountryCode.toUpperCase() : undefined;
        // Only `es-419` is supported in browsers - if any other languages are using `419` we filter them out.
        if (countryCode === '419' && languageCode !== 'es') {
            return undefined;
        }
        var quality = matchedQuality === undefined ? '1' : Number.parseFloat(matchedQuality).toString(); // Remove trailing zeros.
        var locale = countryCode ? "".concat(languageCode, "-").concat(countryCode) : undefined;
        return { languageCode: languageCode, locale: locale, quality: quality };
    };
    /**
     * Get a match from a data object.
     *
     * @param dataObject - A data object.
     *
     * @returns A match or undefined when there is no match.
     */
    LookupList.prototype.getMatch = function (dataObject) {
        var dataObjectEntries = Object.entries(dataObject);
        return dataObjectEntries.length === 0
            ? undefined
            : dataObjectEntries.sort().reverse()[0][1].values().next().value;
    };
    return LookupList;
}());
export default LookupList;
